*===============================================================================
* Project: Bulgaria Poverty Program
* Description: Harmonize HBS data for statistical matching
* Data: HBS 2019 and EU-SILC 2020
* Written by: Britta Rude (Poverty and Equity GP, World Bank)
* Last update: February 2023

* Important remarks: We follow the ECAPOV harmonization for income and energy  
*===============================================================================

***Define year of study
global year 2019
version 16
set seed     292246
set sortseed 857566

*==============================================================================*
* Define directories
*==============================================================================*

clear all

global root "C:/WBG/rep-checks/RR_BGR_2024_93"
global path "${root}/code"
global data "${root}/data"
global tables "$path\Results\Energy\Tables\\${year}"
global figures "$path\Results\Energy\Figures\\${year}"
global dos "$path\Do"

sysdir set PLUS "${dos}/ado"

set scheme s1color





*==============================================================================*
* Upload non-harmonized data 
*==============================================================================*

***Prepare HBS for harmonization and create some descriptive graphs and tables 
do "$dos/01_01_Matching_Harmonize_HBS"

***Prepare EU-SILC for harmonization and create some descriptive graphs and tables 
do "$dos/01_02_Matching_Harmonize_EUSILC.do"

***Combine harmonized data and create summary table  
do "$dos/01_03_Matching_Finaldataset.do"

***Estimate potential matching models and analyze results
do "$dos/01_04_Matching_DifferentModelst.do"

***Regression Tables for best models
do "$dos/01_04_Matching_DifferentModelst_Tests.do"

***Final model and dataset
do "$dos/01_05_Matching_FinalModel.do"

***Final figure
do "$dos/02_Figure9.do"

